@@ -19,6 +19,9 @@
 #include "xaawrap.h"
 #include "xf86fbman.h"
 #include "servermd.h"
+#ifdef COMPOSITE
+#include "cw.h"
+#endif
 
 #define MAX_PREALLOC_MEM	65536	/* MUST be >= 1024 */
 
@@ -39,9 +42,12 @@ static int  XAASetDGAMode(int index, int num, DGADevicePtr devRet);
 static void XAAEnableDisableFBAccess (int index, Bool enable);
 static Bool XAAChangeWindowAttributes (WindowPtr pWin, unsigned long mask);
 
-static DevPrivateKey XAAScreenKey = &XAAScreenKey;
-static DevPrivateKey XAAGCKey = &XAAGCKey;
-static DevPrivateKey XAAPixmapKey = &XAAPixmapKey;
+static int XAAScreenKeyIndex;
+static DevPrivateKey XAAScreenKey = &XAAScreenKeyIndex;
+static int XAAGCKeyIndex;
+static DevPrivateKey XAAGCKey = &XAAGCKeyIndex;
+static int XAAPixmapKeyIndex;
+static DevPrivateKey XAAPixmapKey = &XAAPixmapKeyIndex;
 
 DevPrivateKey XAAGetScreenKey(void) {
     return XAAScreenKey;
@@ -59,7 +65,7 @@ DevPrivateKey XAAGetPixmapKey(void) {
 static Bool SwitchedOut = FALSE;
 
 XAAInfoRecPtr
-XAACreateInfoRec()
+XAACreateInfoRec(void)
 {
     XAAInfoRecPtr infoRec;
 
@@ -70,7 +76,7 @@ XAACreateInfoRec()
     return infoRec;
 }
 
-void 
+void
 XAADestroyInfoRec(XAAInfoRecPtr infoRec)
 {
     if(!infoRec) return;
@@ -88,7 +94,7 @@ XAADestroyInfoRec(XAAInfoRecPtr infoRec)
 }
 
 
-Bool 
+Bool
 XAAInit(ScreenPtr pScreen, XAAInfoRecPtr infoRec)
 {
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
